@@ -89,14 +89,16 @@
     (assert (equal (coerce-name s) (primary-system-name s)))
     (nest
      (if-let ((pathname (first (input-files o s)))))
+     (let ((readtable *readtable*) ;; save outer syntax tables. TODO: proper syntax-control
+           (print-pprint-dispatch *print-pprint-dispatch*)))
      (with-standard-io-syntax)
      (let ((*print-readably* nil)
            ;; Note that our backward-compatible *readtable* is
            ;; a global readtable that gets globally side-effected. Ouch.
            ;; Same for the *print-pprint-dispatch* table.
            ;; We should do something about that for ASDF3 if possible, or else ASDF4.
-           (*readtable* *readtable*)
-           (*print-pprint-dispatch* *print-pprint-dispatch*)
+           (*readtable* readtable) ;; restore inside syntax table
+           (*print-pprint-dispatch* print-pprint-dispatch)
            (*package* (find-package :asdf-user))
            (*default-pathname-defaults*
             ;; resolve logical-pathnames so they won't wreak havoc in parsing namestrings.
@@ -240,7 +242,7 @@ PREVIOUS-TIME when not null is the time at which the PREVIOUS system was loaded.
           (let ((o (make-operation 'define-op)))
             (multiple-value-bind (stamp done-p)
                 (compute-action-stamp plan o system)
-              (return (and (stamp<= stamp (component-operation-time o system))
+              (return (and (timestamp<= stamp (component-operation-time o system))
                            done-p)))))
       (system-out-of-date () nil)))
 
@@ -267,7 +269,7 @@ PREVIOUS-TIME when not null is the time at which the PREVIOUS system was loaded.
                               (pathname-equal
                                (physicalize-pathname pathname)
                                (physicalize-pathname previous-pathname))))
-                     (stamp<= stamp previous-time)
+                     (timestamp<= stamp previous-time)
                      ;; TODO: check that all dependencies are up-to-date.
                      ;; This necessitates traversing them without triggering
                      ;; the adding of nodes to the plan.
